/*
 * $Id: sslclient.c,v 1.3 2001/10/11 05:10:43 carzanig Exp carzanig $
 */
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>

#include <openssl/ssl.h>

#define S_PORT	1111
#define S_HOST	"localhost"

#define BUFSIZE	4096

#define CHK_NULL(x) if ((x)==NULL) exit (1)
#define CHK_ERR(err,s) if ((err)<0) { perror(s); exit(1); }
#define CHK_SSL_ERR(err) if((err)<0){ERR_print_errors_fp(stderr);exit(2);}
/*
 * client request
 */
const char * Req = "ciao mamma!\n";

int connect_host_port(const char* host, unsigned short port)
{
    int fd;
    struct sockaddr_in addr;
    struct sockaddr *mapped_addr;

    fd = socket(PF_INET, SOCK_STREAM, 0);
    if (fd < 0) return -1;
  
    memset(&addr, 0, sizeof(addr));
    addr.sin_family = AF_INET;
    addr.sin_port = htons(port);
    if (isdigit((int)*host)) {			/* if the first char of host */
	addr.sin_addr.s_addr = inet_addr(host); /* is a digit, then assumes  */
    } else {					/*  it is the IP address     */
	struct hostent *hostEntry;
	if ((hostEntry = gethostbyname(host)) != 0) {
	    memcpy(&addr.sin_addr, hostEntry->h_addr, hostEntry->h_length);
	} else {
	    close(fd);
	    return -1;
	}
    }
    if (connect(fd, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
	close(fd);
	return -1;
    }
    return fd;
}

int main(int argc, char *argv[]) 
{
    SSL_CTX *		context		= NULL;
    SSL_METHOD *	method		= NULL;
    SSL *		connection	= NULL;

    int			sock_fd		= -1;
    int			res		= -1;
    char		buf[BUFSIZE];
    /*
     * SSL initialization
     */
    SSL_library_init();
    SSL_load_error_strings();
    OpenSSL_add_ssl_algorithms();
    method = SSLv3_client_method();
    context = SSL_CTX_new(method);
    /*
     * TCP connection setup
     */
    sock_fd = connect_host_port(S_HOST, S_PORT);	CHK_ERR(sock_fd, 
								"connect");
    printf("+ established TCP connection\n");
    /*
     * SSL connection setup
     */
    connection = SSL_new(context);			CHK_NULL(connection);
    res = SSL_set_fd(connection, sock_fd);		CHK_SSL_ERR(res);
    res = SSL_connect(connection);			CHK_SSL_ERR(res);
    printf("+ performed SSL Handshake Protocol\n");
    /*
     * Data 
     */
    res = SSL_write(connection, Req, strlen(Req));	CHK_SSL_ERR(res);
    printf("+ my request:\n%s", Req);
    res = SSL_read(connection, buf, BUFSIZE - 1);	CHK_SSL_ERR(res);
    buf[res] = '\0';
    printf("+ server reply:\n%s", buf);
    /*
     * shutdown and cleanup
     */
    SSL_shutdown(connection);
    close(sock_fd);
    SSL_free(connection);
    SSL_CTX_free(context);

    return 0;
}

